/* GMP3 - A front end for mpg123
 * Copyright (C) 1998 Brett Kosinski
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include <gtk/gtk.h>
#include <glib.h>
#include "albumnlist.h"
#include "playlist.h"
#include "common.h"

/******************************/
/* albumnType management code */
/******************************/

albumnType *newAlbumn(char *name)
{
  albumnType *newAlbumn;
  
  newAlbumn = (albumnType *)malloc(sizeof(albumnType));
  
  newAlbumn->albumnName = (char *)malloc(strlen(name)+2);
  strcpy(newAlbumn->albumnName,name);
  newAlbumn->songList = g_list_alloc();

  return newAlbumn;
}

/*******************************/
/* Albumn List management code */
/*******************************/

/*  
   The code for the albumns is virtually identical to that of the
   regular song list code.  But, I had to make these functions as the
   data types are just a little too different to make generic code to
   manage the lists.
*/   

void constructAlbumnList(GtkWidget *list, GList *albumns)
{
  char *albumnName;
  GList *tempList = g_list_first(albumns)->next;
  GtkWidget *label;
  GtkWidget *listItem;  
  
  while (tempList != NULL)
    {
      albumnName = ((albumnType *)tempList->data)->albumnName;

      listItem = gtk_list_item_new();      
      gtk_widget_show(listItem);                  

      label = gtk_label_new(albumnName);      
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
      gtk_widget_show(label);
      gtk_container_add(GTK_CONTAINER(listItem), label);
      
      gtk_object_set_data(GTK_OBJECT(listItem), list_item_key, 
                          albumnName);
                          
      gtk_container_add(GTK_CONTAINER(list), listItem);                          
                                            
      tempList = tempList->next;
    }
}

/*
*/  

GList *findInAlbumnList(GList *list, char *fileName)
{
  GList *tmpList;
  
  tmpList = g_list_first(list)->next;

  if (tmpList == NULL)
    return NULL;
  
  while (tmpList != NULL)
    {          
      if (strcmp(((albumnType *)tmpList->data)->albumnName, fileName) == 0)
        break;
     
      tmpList = tmpList->next;
    }
  
  return ((albumnType *)tmpList->data)->songList;
}

void updateAlbumnList(GtkWidget *list, GList *albumnList)
{
  gtk_list_clear_items(GTK_LIST(list), 0, -1);  
  constructAlbumnList(list, albumnList);
}

void addToAlbumnList(GList *list, albumnType *albumn)
{
  g_list_append(list, albumn);
}

void removeFromAlbumnList(GList *list, char *name)
{
  GList *tmpList;
  
  tmpList = g_list_first(list)->next;

  if (tmpList == NULL)
    return;
  
  while (tmpList != NULL)
    {    
      if (strcmp(((albumnType *)tmpList->data)->albumnName, name) == 0)
        break;
     
      tmpList = tmpList->next;
    }
  
  g_list_remove(list, tmpList->data);  
}
